
/**
 * A simple example of a an (slightly) ambigious grammar, which is LL(2).
 * The problem is a common prefix of length one.
 *
 * Solutions:
 *   1. Increase lookahead, that is, use option k = 2. 
 *   2. Left factor.
 *   3. Syntactic predicate (arbitrary backtracking).
 *   4. Semantic predicate to distuingish the two choices. An explicit lookup
 *      of the second LA-token is sufficient.
 */
class LL2Parser extends Parser;
options {
    // Solution 1:
    k = 2;
}
{ 
    public static void main(String args[]) throws Exception {
        final LL2Parser parser = 
            new LL2Parser(new SimpleLexer(System.in));
        parser.g();
    }
}

g       : abOrAc EOF
        ;
abOrAc  : ("a" "b"
        | "a" "c")
        EOF
        ;

// Solution 2: Left factorization.
// g   : "a" ("b" | "c") EOF
//     ;

// Solution 3: Syntactic predicate on an LL(3)-production.
// abOrAc  : ("a" "b") => "a" "b"
//         | "a" "c"
//         ;
// Solution 4: Semantic predicate (explicit syntactic predicate):
// abOrAc : {LA(2) == LITERAL_b }? "a" "b"
//         | "a" "c"
//         ;


class SimpleLexer extends Lexer;

WS      : (' '
          | '\t'
          | 
          ( ('\r' '\n') => '\r' '\n'
          | '\r'
          | '\n'
          ) { newline(); })+
        {$setType(Token.SKIP);}
    ;

OTHER   : ('a' .. 'z')+
        ;
